@@ -1,8 +1,12 @@
 # revision history
 # 2009-09-16: added ... argument to print.summary.powerTransform. J. Fox
+# 2015-02-02: added 'gamma' argument to get transformation of (U + gamma)
+# 2015-08-10: added estimateTransform as a generic function
+# 2015-08-24: made 'family' an explicit argument to powerTransformation to clairfy man page.
 
 ### Power families:
-basicPower <- function(U,lambda) {
+basicPower <- function(U,lambda, gamma=NULL) {
+ if(!is.null(gamma)) basicPower(t(t(as.matrix(U) + gamma)), lambda) else{
  bp1 <- function(U,lambda){
   if(any(U[!is.na(U)] <= 0)) stop("First argument must be strictly positive.")
   if (abs(lambda) <= 1.e-6) log(U) else (U^lambda)
@@ -17,9 +21,10 @@ basicPower <- function(U,lambda) {
            paste(colnames(out)[j], round(lambda[j], 2), sep="^")}
     out}  else
     bp1(out, lambda)
-  out}
+  out}}
   
-bcPower <- function(U, lambda, jacobian.adjusted=FALSE) {
+bcPower <- function(U, lambda, jacobian.adjusted=FALSE, gamma=NULL) {
+ if(!is.null(gamma)) bcPower(t(t(as.matrix(U) + gamma)), lambda, jacobian.adjusted) else{
  bc1 <- function(U, lambda){
   if(any(U[!is.na(U)] <= 0)) stop("First argument must be strictly positive.")
   z <- if (abs(lambda) <= 1.e-6) log(U) else ((U^lambda) - 1)/lambda
@@ -34,7 +39,7 @@ bcPower <- function(U, lambda, jacobian.adjusted=FALSE) {
     colnames(out) <- paste(colnames(out), round(lambda, 2), sep="^")
     out}  else
     bc1(out, lambda)
-  out}
+  out}}
   
 yjPower <- function(U, lambda, jacobian.adjusted=FALSE) {
  yj1 <- function(U, lambda){
@@ -59,17 +64,17 @@ yjPower <- function(U, lambda, jacobian.adjusted=FALSE) {
   
 powerTransform <- function(object, ...) UseMethod("powerTransform")
 
-powerTransform.default <- function(object, ...) {
+powerTransform.default <- function(object, family="bcPower", ...) {
    y <- object
    if(!inherits(y, "matrix") & !inherits(y, "data.frame")) {
        y <- matrix(y,ncol=1)
        colnames(y) <- c(paste(deparse(substitute(object))))}
    y <- na.omit(y)
    x <- rep(1, dim(y)[1]) 
-   estimateTransform(x, y, NULL, ...)
+   estimateTransform(x, y, NULL, family=family, ...)
    }                                    
 
-powerTransform.lm <- function(object,...) {
+powerTransform.lm <- function(object, family="bcPower", ...) {
     mf <- if(is.null(object$model)) 
             update(object, model=TRUE, method="model.frame")$model 
             else object$model
@@ -81,10 +86,10 @@ powerTransform.lm <- function(object,...) {
         x <- matrix(rep(1,dim(mf)[1]), ncol=1) }
     else {
         x <- model.matrix(mt, mf, contrasts)   } 
-  estimateTransform(x, y, w, ...)
+  estimateTransform(x, y, w, family=family, ...)
   }                                                 
   
-powerTransform.formula <- function(object, data, subset, weights, na.action,
+powerTransform.formula <- function(object, data, subset, weights, na.action, family="bcPower",
   ...) {
     mf <- match.call(expand.dots = FALSE)
     m <- match(c("object", "data", "subset", "weights", "na.action"), 
@@ -102,65 +107,72 @@ powerTransform.formula <- function(object, data, subset, weights, na.action,
         x <- matrix(rep(1, dim(mf)[1]), ncol=1) }
     else {
         x <- model.matrix(mt, mf)   } 
-  estimateTransform(x, y, w, ...)
+  estimateTransform(x, y, w, family=family, ...)
   } 
 
 estimateTransform <- function(X, Y, weights=NULL,
-   family="bcPower", start=NULL, method="L-BFGS-B", ...) {
-   fam <- match.fun(family)
-   Y <- as.matrix(Y) # coerces Y to be a matrix.
-   X <- as.matrix(X) # coerces X to be a matrix. 
-   w <- if(is.null(weights)) 1 else sqrt(weights)
-   nc <- dim(Y)[2]
-   nr <- nrow(Y)
-   xqr <- qr(w * X)       
-   llik <- function(lambda){
-        (nr/2)*log(((nr - 1)/nr) *
-               det(var(qr.resid(xqr, w*fam(Y, lambda, j=TRUE)))))
-        }
-   llik1d <- function(lambda,Y){
-        (nr/2)*log(((nr - 1)/nr) * var(qr.resid(xqr, w*fam(Y, lambda, j=TRUE))))
-        }
-   if (is.null(start)) {
-        start <- rep(1, nc)
-        for (j in 1:nc){
-            res<- suppressWarnings(optimize(
-                    f = function(lambda) llik1d(lambda,Y[ , j, drop=FALSE]),
-                    lower=-3, upper=+3))
-            start[j] <- res$minimum
-            }
-        }
-   res<-optim(start, llik, hessian=TRUE, method=method,  ...)
-   if(res$convergence != 0)
-        warning(paste("Convergence failure: return code =", res$convergence))
-   res$start<-start
-   res$lambda <- res$par
-   names(res$lambda) <- 
-     if (is.null(colnames(Y))) paste("Y", 1:dim(Y)[2], sep="")
-     else colnames(Y)
-   roundlam <- res$lambda
-   stderr <- sqrt(diag(solve(res$hessian)))
-   lamL <- roundlam - 1.96 * stderr
-   lamU <- roundlam + 1.96 * stderr
-   for (val in rev(c(1, 0, -1, .5, .33, -.5, -.33, 2, -2))) { 
-       sel <- lamL <= val & val <= lamU 
-    roundlam[sel] <- val
+                              family="bcPower", start=NULL, method="L-BFGS-B", ...) {
+  if(family == "skewPower") 
+    estimateTransform.skewPower(X, Y, weights,  ...) else
+      estimateTransform.default(X, Y, weights, family, start, method, ...)
+}
+
+# estimateTransform.default is renamed 'estimateTransform
+estimateTransform.default <- function(X, Y, weights=NULL,
+                                      family="bcPower", start=NULL, method="L-BFGS-B", ...) {
+  fam <- match.fun(family)
+  Y <- as.matrix(Y) # coerces Y to be a matrix.
+  X <- as.matrix(X) # coerces X to be a matrix. 
+  w <- if(is.null(weights)) 1 else sqrt(weights)
+  nc <- dim(Y)[2]
+  nr <- nrow(Y)
+  xqr <- qr(w * X)       
+  llik <- function(lambda){
+    (nr/2)*log(((nr - 1)/nr) *
+                 det(var(qr.resid(xqr, w*fam(Y, lambda, j=TRUE, ...)))))
+  }
+  llik1d <- function(lambda,Y){
+    (nr/2)*log(((nr - 1)/nr) * var(qr.resid(xqr, w*fam(Y, lambda, j=TRUE, ...))))
+  }
+  if (is.null(start)) {
+    start <- rep(1, nc)
+    for (j in 1:nc){
+      res<- suppressWarnings(optimize(
+        f = function(lambda) llik1d(lambda,Y[ , j, drop=FALSE]),
+        lower=-3, upper=+3))
+      start[j] <- res$minimum
     }
-   res$roundlam <- roundlam
-   res$par <- NULL
-   res$family<-family
-   res$xqr <- xqr
-   res$y <- Y
-   res$x <- X    
-   res$weights <- weights
-   class(res) <- "powerTransform"
-   res
-   }
+  }
+  res<-optim(start, llik, hessian=TRUE, method=method,  ...)
+  if(res$convergence != 0)
+    warning(paste("Convergence failure: return code =", res$convergence))
+  res$start<-start
+  res$lambda <- res$par
+  names(res$lambda) <- 
+    if (is.null(colnames(Y))) paste("Y", 1:dim(Y)[2], sep="")
+  else colnames(Y)
+  roundlam <- res$lambda
+  stderr <- sqrt(diag(solve(res$hessian)))
+  lamL <- roundlam - 1.96 * stderr
+  lamU <- roundlam + 1.96 * stderr
+  for (val in rev(c(1, 0, -1, .5, .33, -.5, -.33, 2, -2))) { 
+    sel <- lamL <= val & val <= lamU 
+    roundlam[sel] <- val
+  }
+  res$roundlam <- roundlam
+  res$par <- NULL
+  res$family<-family
+  res$xqr <- xqr
+  res$y <- Y
+  res$x <- as.matrix(X)    
+  res$weights <- weights
+  class(res) <- "powerTransform"
+  res
+}
    
 testTransform <- function(object, lambda) UseMethod("testTransform")   
    
-testTransform.powerTransform <- function(object, 
-  lambda=rep(1, dim(object$y)[2])){
+testTransform.powerTransform <- function(object, lambda=rep(1, dim(object$y)[2])){
    fam <- match.fun(object$family)
    Y <- cbind(object$y) # coerces Y to be a matrix.
    nc <- dim(Y)[2]
@@ -170,7 +182,7 @@ testTransform.powerTransform <- function(object,
    w <- if(is.null(object$weights)) 1 else sqrt(object$weights)  
    llik <- function(lambda){
         (nr/2) * log(((nr - 1)/nr) * 
-             det(var(qr.resid(xqr, w*fam(Y, lam,j=TRUE)))))
+             det(var(qr.resid(xqr, w * fam(Y, lam, jacobian.adjusted=TRUE)))))
         }
    LR <- 2 * (llik(lambda) - object$value)
    df <- length(object$lambda)
